const createExpressDriver = (
  {
    express,
    middlewares = [],
    port = 3000,
  }
) => {
  var server = express()

  middlewares.forEach(mw => server.use(mw))

  const onCreate = (endpoint, createFn) => {
    server.post(`/${endpoint}`, function({ body, query }, res) {
      createFn(query, body, result => res.send(result))
    })
  }

  const onRead = (endpoint, readFn) => {
    server.get(`/${endpoint}/:id`, function({ query, params }, res) {
      readFn(params.id, query, result => res.send(result))
    })
  }

  const onList = (endpoint, readFn) => {
    server.get(`/${endpoint}`, function({ query }, res) {
      readFn(query, result => res.send(result))
    })
  }

  const onUpdate = (endpoint, updateFn) => {
    server.patch(`/${endpoint}/:id`, function({ body, query, params }, res) {
      updateFn(params.id, query, body, result => res.send(result))
    })
  }

  const onDelete = (endpoint, deleteFn) => {
    server.delete(`/${endpoint}/:id`, function({ query, params }, res) {
      deleteFn(params.id, query, result => res.send(result))
    })
  }

  server.listen(port)

  return {
    onCreate,
    onRead,
    onList,
    onUpdate,
    onDelete,
  }
}

module.exports = createExpressDriver
